// Generated by CoffeeScript 1.8.0
var _Core, _Dialog, _Document, _Editor, _Events, _On, _Plugin, _Terminal, _Utility,
  __slice = [].slice;

_Core = (function() {
  function _Core() {
    this.currentPlugin = null;
    this.currentDocument = new _Document;
    this.version = "1.4 (Build 2652)";
  }

  _Core.prototype.coffee = function(code, bare, exec) {
    var error, result;
    if (bare == null) {
      bare = false;
    }
    if (exec == null) {
      exec = false;
    }
    try {
      if (typeof code === 'object') {
        code = eval(code)[0];
      }
      result = CoffeeScript.compile(code, {
        bare: bare
      });
      if (exec) {
        return eval.apply(window, [result]);
      } else {
        return result;
      }
    } catch (_error) {
      error = _error;
      return $Terminal.echo("Coffee:: Error : " + (error.toString()));
    }
  };

  _Core.prototype.less = function(code, name) {
    var error, parser;
    try {
      if (typeof code === 'object') {
        code = eval(code)[0];
      }
      if (name != null) {
        code = code.replace(new RegExp("(\.[a-z]+)", "gm"), ".ace_$1");
        code = code.replace(new RegExp("(#" + name + ")", "gm"), ".ace-" + name);
      }
      parser = window.less.Parser();
      parser.parse(code, (function(_this) {
        return function(error, result) {
          if (!error) {
            return result.toCSS();
          }
        };
      })(this));
    } catch (_error) {
      error = _error;
      $Terminal.echo("LESS:: Error : " + (error.toString()));
    }
    return "";
  };

  _Core.prototype.registerPluginMenu = function() {
    return core.registerPluginMenu_(JSON.stringify(Object.keys(this.currentPlugin.menu())));
  };

  _Core.prototype.loadPlugin = function(name, path, source) {
    if (typeof code === 'object') {
      source = eval(code)[0];
    }
    this.coffee("" + source + "\n\nwindow." + name + " = " + name + " unless window." + name + ";", false, true);
    this.currentPlugin = new window[name](name, path);
    this.registerPluginMenu();
    return this.currentPlugin.init();
  };

  _Core.prototype.haltPlugin = function() {
    if (this.currentPlugin != null) {
      this.currentPlugin.halt();
      $Events.removeById(this.currentPlugin.name);
      window[this.currentPlugin.name] = null;
      return this.currentPlugin = null;
    }
  };

  _Core.prototype.runPlugin = function(name, path, source) {
    if ((this.currentPlugin != null) && this.currentPlugin.name !== name) {
      this.haltPlugin();
    }
    if ((this.currentPlugin == null) || this.currentPlugin.name !== name) {
      this.loadPlugin(name, path, source);
    }
    return this.currentPlugin.run();
  };

  _Core.prototype.testPlugin = function(refresh) {
    var pluginContent, pluginName, pluginPath;
    if (refresh == null) {
      refresh = false;
    }
    pluginName = $Document.info().filename.replace($Document.info().extension, "").replace(".", "");
    pluginPath = $Document.info().folder;
    pluginContent = $Document.info().content;
    if (refresh) {
      this.haltPlugin();
    }
    return this.runPlugin(pluginName, pluginPath, pluginContent);
  };

  return _Core;

})();

_Events = (function() {
  function _Events() {
    this.list = {
      documentChanged: {},
      documentSaved: {},
      cursorChanged: {},
      callback: {}
    };
  }

  _Events.prototype.exec = function(event, id, args) {
    var callback, eventId, _ref, _results;
    _ref = this.list[event];
    _results = [];
    for (eventId in _ref) {
      callback = _ref[eventId];
      if (id == null) {
        _results.push(callback(args));
      } else {
        if (id === eventId) {
          _results.push(callback(args));
        } else {
          _results.push(void 0);
        }
      }
    }
    return _results;
  };

  _Events.prototype.add = function(event, id, callback) {
    if (id == null) {
      id = pp.uuid();
    }
    this.list[event][id] = callback;
    return "$Events.exec('callback','" + id + "','%@')";
  };

  _Events.prototype.remove = function(event, id) {
    return delete this.list[event][id];
  };

  _Events.prototype.removeById = function(id) {
    var callback, content, event, eventId, _ref, _results;
    _ref = this.list;
    _results = [];
    for (event in _ref) {
      content = _ref[event];
      _results.push((function() {
        var _ref1, _results1;
        _ref1 = this.list[event];
        _results1 = [];
        for (eventId in _ref1) {
          callback = _ref1[eventId];
          if (id === eventId) {
            _results1.push(this.remove(event, id));
          } else {
            _results1.push(void 0);
          }
        }
        return _results1;
      }).call(this));
    }
    return _results;
  };

  _Events.prototype.set = function(event, id, callback) {
    if (callback != null) {
      return this.add(event, id, callback);
    } else {
      return this.remove(event, id, callback);
    }
  };

  return _Events;

})();


/*
@class Dialog
@description The Dialog object - used to invoke dialogs
 */

_Dialog = (function() {
  function _Dialog() {}


  /*
  @method alert
  @description Display an alert dialog
  @param title|required|string|The main dialog title
  @param info|optional|string|Related information
  @param callback|optional|function|The function to be called, when the dialog closes
  @example $Dialog.alert("Finished!","The operation has finished")
   */

  _Dialog.prototype.alert = function(title, info, callback) {
    var event;
    if (info == null) {
      info = "";
    }
    if (callback != null) {
      event = $Events.set("callback", null, callback);
    } else {
      event = null;
    }
    return core.dialogAlert_info_callback_(title, info, event);
  };


  /*
  @method input
  @description Show an input dialog and get user input
  @param title|required|string|The title/prompt of the dialog
  @param callback|required|function|The function to be called, with the input, when the dialog closes
  @return null|
  @example $Dialog.input("Enter a number:", (number)=>
  @example    $Terminal.echo("Got : #{ number }");    
  @example );
   */

  _Dialog.prototype.input = function(title, callback) {
    var event;
    event = $Events.set("callback", null, callback);
    return core.dialogInput_callback_(title, event);
  };

  return _Dialog;

})();


/*
@class Document
@description The current Document object
@property label|string|The document's label as seen in Peppermint
@property path|string|The full path of the document
@property filename|string|The document's pure filename
@property folder|string|The document's pure folder name
@property extension|string|The document's file extension
 */

_Document = (function() {
  function _Document() {
    this.content = null;
  }


  /*
  @method info
  @description Get/Update Info of current document and/or create a temporary copy for processing
  @param ext|optional|string|Force a particular extension (if document is to be created)
  @return Document|A copy of the updated Document object, with all its properties populated
  @example path = $Document.info().path
   */

  _Document.prototype.info = function(ext) {
    if (ext == null) {
      ext = "tmp";
    }
    if ((this.content == null) || pp.getContent() !== this.content || core.getLabel() !== this.label) {
      this.content = pp.getContent();
      this.label = core.getLabel();
      this.path = pp.getFullPath();
      if (this.path === "") {
        this.path = core.getTempFile_withExtension_(this.content, ext);
      }
      this.filename = this.path.replace(/^.*[\\\/]/, '');
      this.folder = this.path.replace(this.filename, '');
      this.extension = /(?:\.([^.]+))?$/.exec(this.filename)[1];
    }
    return this;
  };


  /*
  @method temp
  @description Create temporary document instance with contents
  @param content|required|string|The document's content
  @return Document|A Document object, with all its properties populated
  @example temp = $Document.temp("my contents");
  @example path = temp.path;
   */

  _Document.prototype.temp = function(content) {
    this.path = core.getTempFile_withExtension_(content, "tmp");
    this.content = content;
    this.filename = this.path.replace(/^.*[\\\/]/, '');
    this.label = this.filename;
    this.folder = this.path.replace(this.filename, '');
    this.extension = /(?:\.([^.]+))?$/.exec(this.filename)[1];
    return this;
  };

  return _Document;

})();


/*
@class Editor
@description The main Event-handling object
 */

_Editor = (function() {
  function _Editor() {}


  /*
  @method bind
  @description Create or Destroy and editor Key Binding
  @param key|required|string|Key combination
  @param callback|optional|function|Callback on key combination (null to destroy)
  @return null|
  @example $Editor.bind("Cmd+R", ()=>
  @example    $Terminal.echo("Running");
  @example );
   */

  _Editor.prototype.bind = function(key, callback) {
    if (callback == null) {
      callback = null;
    }
    return pp.editor.commands.bindKey(key, callback);
  };


  /*
  @method content
  @description Get/Set editor content
  @param value|optional|string|New editor content
  @param undoable|optional|boolean|Set if change will be undoable (default: true)
  @return null or string|The content of the editor
  @example currentContent = $Editor.content();
  @example 
  @example # Let's change it 
  @example $Editor.content("New content");
   */

  _Editor.prototype.content = function(value, undoable) {
    if (undoable == null) {
      undoable = true;
    }
    if (value != null) {
      if (undoable) {
        return pp.setUndoableContent(value);
      } else {
        return pp.setContent(value);
      }
    } else {
      return pp.getContent();
    }
  };


  /*
  @method selection
  @description Get/Set current editor selection
  @param value|optional|string|Replace current selection with value
  @return null or string|The currently selected text
  @example currentContent = $Editor.content();
  @example 
  @example # Let's change it 
  @example $Editor.content("New content");
   */

  _Editor.prototype.selection = function(value) {
    if (value != null) {
      return pp.setSelection(value);
    } else {
      return pp.getSelection();
    }
  };

  _Editor.prototype.option = {

    /*
    @method option.gutter
    @description Get/Set Gutter state
    @param onoff|optional|boolean|True/False
    @return null or boolean|
     */
    gutter: function(onoff) {
      if (onoff != null) {
        return pp.toggleGutter(onoff);
      } else {
        return pp.editor.renderer.getShowGutter();
      }
    },

    /*
    @method option.printMargin
    @description Get/Set Print Margin state
    @param onoff|optional|boolean|True/False
    @return null or boolean|
     */
    printMargin: function(onoff) {
      if (onoff != null) {
        return pp.togglePrintMargin(onoff);
      } else {
        return pp.editor.getShowPrintMargin();
      }
    },

    /*
    @method option.indentGuides
    @description Get/Set Indentation Guides state
    @param onoff|optional|boolean|True/False
    @return null or boolean|
     */
    indentGuides: function(onoff) {
      if (onoff != null) {
        return pp.toggleIndentationGuides(onoff);
      } else {
        return pp.editor.renderer.getDisplayIndentGuides();
      }
    },

    /*
    @method option.invisibles
    @description Get/Set Invisibles state
    @param onoff|optional|boolean|True/False
    @return null or boolean|
     */
    invisibles: function(onoff) {
      if (onoff != null) {
        return pp.toggleInvisibles(onoff);
      } else {
        return pp.editor.getShowInvisibles();
      }
    },

    /*
    @method option.softTabs
    @description Get/Set Soft Tabs state
    @param onoff|optional|boolean|True/False
    @return null or boolean|
     */
    softTabs: function(onoff) {
      if (onoff != null) {
        return pp.toggleSoftTabs(onoff);
      } else {
        return pp.editor.getUseSoftTabs();
      }
    },

    /*
    @method option.wordWrap
    @description Get/Set Word Wrap state
    @param onoff|optional|boolean|True/False
    @return null or boolean|
     */
    wordWrap: function(onoff) {
      if (onoff != null) {
        return pp.toggleWordWrap(onoff);
      } else {
        return pp.editor.getSession().getUseWrapMode();
      }
    },

    /*
    @method option.tabSize
    @description Get/Set Tab Size
    @param size|optional|number|Tab size in characters
    @return null or number|
     */
    tabSize: function(size) {
      if (size != null) {
        return pp.setTabSize(size);
      } else {
        return pp.editor.getSession().getTabSize();
      }
    },

    /*
    @method option.wrapLimit
    @description Get/Set Wrap Limit
    @param size|optional|number|Wrap Limit in characters
    @return null or number|
     */
    wrapLimit: function(limit) {
      if (limit != null) {
        return pp.setWrapLimit(limit);
      } else {
        return pp.editor.getSession().getWrapLimitRange().min;
      }
    }
  };

  return _Editor;

})();


/*
@class On
@description The main Event-handling object
 */

_On = (function() {
  function _On(pluginName) {
    this.pluginId = pluginName;
  }


  /*
  @method documentChanged
  @description Set callback to be executed, when document changes
  @param callback|required|function|The callback to be executed, or null to remove handler
  @return null|
  @example init: ()->
  @example    $$x = 1;
  @example    $$on.documentChanged(()=>
  @example        $Terminal.echo("Document changed");
  @example        $Terminal.echo("x = " + $$x++);
  @example    );
  @example  
  @example halt: ()->
  @example    $$on.documentChanged(null);
   */

  _On.prototype.documentChanged = function(callback) {
    $Events.set("documentChanged", this.pluginId, callback);
    return null;
  };


  /*
  @method documentSaved
  @description Set callback to be executed, when document is saved
  @param callback|required|function|The callback to be executed, or null to remove handler
  @return null|
  @example $$on.documentSaved(()=>
  @example    # do something
  @example )
   */

  _On.prototype.documentSaved = function(callback) {
    $Events.set("documentSaved", this.pluginId, callback);
    return null;
  };


  /*
  @method cursorChanged
  @description Set callback to be executed, when document changes
  @param callback|required|function|The callback to be executed, or null to remove handler
  @return null|
  @example $$on.cursorChanged(()=>
  @example    # do something
  @example )
   */

  _On.prototype.cursorChanged = function(callback) {
    $Events.set("cursorChanged", this.pluginId, callback);
    return null;
  };

  return _On;

})();


/*
@class Plugin
@description The main Inheritable Plugin class
@property name|string|The plugin's name as defined in the <code>info.json</code>
@property path|string|The plugin's folder path
@property on|On|An associated <a href='#api-class-on'>On</a> Object for event handling
 */

_Plugin = (function() {
  function _Plugin(name, path) {
    this.name = name;
    this.path = path;
    this.bin = window.binariesIndex;
    this.on = new _On(this.name);
  }


  /*
  @method init
  @description Called automatically when plugin is initialized the first time
  @return null|
  @example class MyPlugin extends $Plugin
  @example    init:->
  @example        $Terminal.echo("Plugin initialised!");
   */

  _Plugin.prototype.init = function() {};


  /*
  @method run
  @description Called every time the plugin is executed
  @return null|
  @example class MyPlugin extends $Plugin
  @example    init:->
  @example        $Terminal.echo("Plugin initialised!");
  @example    
  @example    run:->
  @example        $Terminal.echo("Plugin is running!");
   */

  _Plugin.prototype.run = function() {};


  /*
  @method menu
  @description Called automatically when plugin is initialized the first time
  @return object|A dictionary of menu items associated with the plugin (Menu Item =&gt; callback)
  @example class MyPlugin extends $Plugin
  @example    init:->
  @example        $Terminal.echo("Plugin initialised!");
  @example    
  @example    doSomething:(arg)->
  @example        $Terminal.echo("Doing something with #{ arg }");
  @example
  @example    menu:-> {
  @example        "First menu item"   :=> @doSomething("first")
  @example        "Second menu item"  :=> @doSomething("second")
  @example        "Third menu item"   :=> @doSomething("third")
  @example    }
   */

  _Plugin.prototype.menu = function() {
    return {};
  };


  /*
  @method halt
  @description Called automatically when plugin is terminated, or another plugin is executed
  @return null|
  @example class MyPlugin extends $Plugin
  @example    init:->
  @example        $Terminal.echo("Plugin initialised!");
  @example    
  @example    halt:->
  @example        $Terminal.echo("Plugin is being terminated...");
   */

  _Plugin.prototype.halt = function() {};


  /*
  @method import
  @description Import a local file's contents
  @param path|required|string|The path to the file
  @param relative|option|boolean|Is the path relative to the plugin folder? (default: true)
  @return string|The file contents
   */

  _Plugin.prototype["import"] = function(path, relative) {
    if (relative == null) {
      relative = true;
    }
    if (relative) {
      path = this.path + "/" + path;
    }
    return core.file_(path);
  };


  /*
  @method include
  @description Include a local script file (JavaScript or CoffeeScript) and execute it in global scope
  @param path|required|string|The path to the file
  @param relative|option|boolean|Is the path relative to the plugin folder? (default: true)
  @return string|The file contents
   */

  _Plugin.prototype.include = function(path, relative) {
    if (relative == null) {
      relative = true;
    }
    if (relative) {
      path = this.path + "/" + path;
    }
    return core.importjs_(path);
  };

  return _Plugin;

})();


/*
@class Terminal
@description The main Terminal object - representing the console panel
@property verbose|boolean|Display more info <i>(Default: false)</i>
 */

_Terminal = (function() {
  function _Terminal() {
    this.environment = {};
    this.verbose = false;
  }


  /*
  @method show
  @description Show the terminal panel, if not already open
  @return null|
  @example $Terminal.show()
   */

  _Terminal.prototype.show = function() {
    pp.showConsoleTab();
    return null;
  };


  /*
  @method clear
  @description Clear the terminal panel
  @return null|
  @example $Terminal.clear()
   */

  _Terminal.prototype.clear = function() {
    terminal.clear();
    return null;
  };


  /*
  @method echo
  @description Echoes given string to the terminal
  @param message|required|string|The message to be displayed
  @param attributes|optional|string|A style attribute string
  @return null|
  @example $Terminal.echo("Hello world!")
   */

  _Terminal.prototype.echo = function(msg, attributes) {
    if (attributes != null) {
      msg = "[[" + attributes + "]" + msg + "]";
    }
    if (typeof msg === 'object') {
      msg = eval(msg)[0];
    }
    terminal.echo(msg);
    return null;
  };


  /*
  @method exec
  @description Asynchronously execute task and print output in terminal
  @param args|required|string...|Command/Arguments to be passed
  @param completed|optional|callback|Callback to be executed when task is completed
  @return string|The execution result
  @example $Terminal.exec("php","--version")
   */

  _Terminal.prototype.exec = function() {
    var arg, args, completed, outp, _i, _len;
    args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    if (this.verbose) {
      outp = "";
      for (_i = 0, _len = args.length; _i < _len; _i++) {
        arg = args[_i];
        if (typeof arg !== 'function') {
          outp += arg + " ";
        }
      }
      $Terminal.echo("# " + outp);
    }
    if (typeof args[args.length - 1] === 'function') {
      completed = args[args.length - 1];
      args.pop();
    } else {
      completed = null;
    }
    args.push((function(_this) {
      return function(output) {
        if (output.trim() !== "") {
          return $Terminal.echo(output);
        }
      };
    })(this));
    if (completed != null) {
      args.push(completed);
    }
    return this.execSilent.apply(this, args);
  };


  /*
  @method execSilent
  @description Execute command in terminal (either synchronously, or async)
  @param args|required|string...|Command/Arguments to be passed
  @param action|optional|callback|Callback to be executed when any data is served
  @param completed|optional|callback|Callback to be executed when task is completed
  @return string|The execution result
  @example $Terminal.execSilent("php","--version")
  @example
  @example # This is pretty much like `$Terminal.exec()`
  @example $Terminal.execSilent("php","myscript.php", (output)=>
  @example    $Terminal.echo("Got output: #{ output }");
  @example ,(result)=>
  @example    $Terminal.echo("Task terminated!");
  @example );
   */

  _Terminal.prototype.execSilent = function() {
    var action, args, completed;
    args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
    action = null;
    completed = null;
    if (typeof args[args.length - 2] === 'function') {
      action = $Events.set("callback", null, args[args.length - 2]);
      completed = $Events.set("callback", null, args[args.length - 1]);
      args.pop();
      args.pop();
    } else {
      if (typeof args[args.length - 1] === 'function') {
        action = $Events.set("callback", null, args[args.length - 1]);
        args.pop();
      }
    }
    args = JSON.stringify(args);
    if (action !== null || completed !== null) {
      window.setTerminalStdin(true);
    }
    if (Object.keys(this.environment).length) {
      return core.terminalExec_env_action_completed_(args, JSON.stringify(this.environment), action, completed);
    } else {
      return core.terminalExec_env_action_completed_(args, "", action, completed);
    }
  };


  /*
  @method setVar
  @description Set custom environment variable
  @param variable|required|string|The name of the variable
  @param value|required|string|The value of the variable
  @return null|
  @example $Terminal.setVar("MYVAR","VALUE");
   */

  _Terminal.prototype.setVar = function(variable, value) {
    return this.environment[variable] = value;
  };

  _Terminal.prototype.write = function(msg) {
    return core.terminalWrite_(msg);
  };


  /*
  @method kill
  @description Kill existing async process being executed
  @return null|
  @example $Terminal.kill();
   */

  _Terminal.prototype.kill = function() {
    return core.terminalKill();
  };

  return _Terminal;

})();


/*
@class Utility
@description The main Utility object - representing the Utility Panel on the right side of the Peppermint window
 */

_Utility = (function() {
  function _Utility() {}


  /*
  @method load
  @description Load html tool in the Utility View
  @param title|required|string|The new title for the Utility View
  @param source|required|string|Path to the .html file to be loaded or html source
  @param base|optional|string|Base path to be used
  @return null|
  @example $Utility.load("My great tool", "#{ @path }/index.html")
   */

  _Utility.prototype.load = function(title, source, base) {
    if (base == null) {
      base = null;
    }
    if (base == null) {
      return core.loadUtility_label_(source, title);
    } else {
      return core.loadHtml_label_withBasePath_(source, title, base);
    }
  };


  /*
  @method loadDocument
  @description Load custom document in the Utility View
  @param title|required|string|The new title for the Utility View
  @param content|required|string|The script's contents
  @param spec|required|string|The spec to be used for highlighting
  @return null|
  @example $Utility.loadDocument("My PHP Script", "<?php echo 'done'; ?>", "php")
   */

  _Utility.prototype.loadDocument = function(title, content, spec) {
    return core.loadEditorUtility_label_mode_(content, title, spec);
  };

  return _Utility;

})();

if (!this.$Core) {
  this.$Core = new _Core;
}

if (!this.$Events) {
  this.$Events = new _Events;
}

if (!this.$Dialog) {
  this.$Dialog = new _Dialog;
}

if (!this.$Document) {
  this.$Document = new _Document;
}

if (!this.$Editor) {
  this.$Editor = new _Editor;
}

if (!this.$Terminal) {
  this.$Terminal = new _Terminal;
}

if (!this.$Utility) {
  this.$Utility = new _Utility;
}

if (!this.$Plugin) {
  this.$Plugin = _Plugin;
}
